/* Generated automatically by the program `gencodes'
   from the machine description file `md'.  */

#ifndef GCC_INSN_CODES_H
#define GCC_INSN_CODES_H

enum insn_code {
  CODE_FOR_nothing = 0,
  CODE_FOR_load_pcrel_symsi = 111,
  CODE_FOR_load_pcrel_symdi = 112,
  CODE_FOR_movsi_lo_sum_pic = 119,
  CODE_FOR_movsi_high_pic = 120,
  CODE_FOR_movsi_pic_gotdata_op = 121,
  CODE_FOR_movdi_lo_sum_pic = 127,
  CODE_FOR_movdi_high_pic = 128,
  CODE_FOR_movdi_pic_gotdata_op = 129,
  CODE_FOR_seth44 = 133,
  CODE_FOR_setm44 = 134,
  CODE_FOR_setl44 = 135,
  CODE_FOR_sethh = 136,
  CODE_FOR_setlm = 137,
  CODE_FOR_sethm = 138,
  CODE_FOR_setlo = 139,
  CODE_FOR_embmedany_sethi = 140,
  CODE_FOR_embmedany_losum = 141,
  CODE_FOR_embmedany_brsum = 142,
  CODE_FOR_embmedany_textuhi = 143,
  CODE_FOR_embmedany_texthi = 144,
  CODE_FOR_embmedany_textulo = 145,
  CODE_FOR_embmedany_textlo = 146,
  CODE_FOR_movdf_cc_v9 = 165,
  CODE_FOR_movdf_cc_reg_sp64 = 166,
  CODE_FOR_extendsfdf2 = 197,
  CODE_FOR_truncdfsf2 = 200,
  CODE_FOR_floatsisf2 = 203,
  CODE_FOR_floatsidf2 = 204,
  CODE_FOR_floatdisf2 = 206,
  CODE_FOR_floatdidf2 = 207,
  CODE_FOR_fix_truncsfsi2 = 209,
  CODE_FOR_fix_truncdfsi2 = 210,
  CODE_FOR_fix_truncsfdi2 = 212,
  CODE_FOR_fix_truncdfdi2 = 213,
   CODE_FOR_adddi3_sp32 = CODE_FOR_nothing,
   CODE_FOR_uaddvdi4_sp32 = CODE_FOR_nothing,
   CODE_FOR_addvdi4_sp32 = CODE_FOR_nothing,
  CODE_FOR_addsi3 = 216,
   CODE_FOR_subdi3_sp32 = CODE_FOR_nothing,
   CODE_FOR_usubvdi4_sp32 = CODE_FOR_nothing,
   CODE_FOR_subvdi4_sp32 = CODE_FOR_nothing,
  CODE_FOR_subsi3 = 232,
  CODE_FOR_muldi3_v8plus = 248,
  CODE_FOR_mulsidi3_v8plus = 250,
  CODE_FOR_const_mulsidi3_v8plus = 251,
   CODE_FOR_const_mulsidi3_sp32 = CODE_FOR_nothing,
  CODE_FOR_const_mulsidi3_sp64 = 253,
  CODE_FOR_smulsi3_highpart_v8plus = 254,
  CODE_FOR_const_smulsi3_highpart_v8plus = 256,
   CODE_FOR_const_smulsi3_highpart = CODE_FOR_nothing,
  CODE_FOR_umulsidi3_v8plus = 257,
   CODE_FOR_const_umulsidi3_sp32 = CODE_FOR_nothing,
  CODE_FOR_const_umulsidi3_sp64 = 259,
  CODE_FOR_const_umulsidi3_v8plus = 260,
  CODE_FOR_umulsi3_highpart_v8plus = 261,
  CODE_FOR_const_umulsi3_highpart_v8plus = 262,
   CODE_FOR_const_umulsi3_highpart = CODE_FOR_nothing,
   CODE_FOR_umulxhi_v8plus = CODE_FOR_nothing,
   CODE_FOR_xmulx_v8plus = CODE_FOR_nothing,
   CODE_FOR_xmulxhi_v8plus = CODE_FOR_nothing,
   CODE_FOR_divsi3_sp32 = CODE_FOR_nothing,
  CODE_FOR_divsi3_sp64 = 266,
  CODE_FOR_divdi3 = 267,
   CODE_FOR_udivsi3_sp32 = CODE_FOR_nothing,
  CODE_FOR_udivsi3_sp64 = 269,
  CODE_FOR_udivdi3 = 270,
  CODE_FOR_anddi3 = 275,
  CODE_FOR_andsi3 = 276,
  CODE_FOR_iordi3 = 279,
  CODE_FOR_iorsi3 = 280,
  CODE_FOR_xordi3 = 283,
  CODE_FOR_xorsi3 = 284,
   CODE_FOR_negdi2_sp32 = CODE_FOR_nothing,
   CODE_FOR_unegvdi3_sp32 = CODE_FOR_nothing,
   CODE_FOR_negvdi3_sp32 = CODE_FOR_nothing,
  CODE_FOR_negsi2 = 300,
  CODE_FOR_one_cmpldi2 = 313,
  CODE_FOR_one_cmplsi2 = 314,
  CODE_FOR_adddf3 = 322,
  CODE_FOR_addsf3 = 323,
  CODE_FOR_subdf3 = 325,
  CODE_FOR_subsf3 = 326,
  CODE_FOR_muldf3 = 328,
  CODE_FOR_mulsf3 = 329,
  CODE_FOR_fmadf4 = 330,
  CODE_FOR_fmsdf4 = 331,
  CODE_FOR_fmasf4 = 334,
  CODE_FOR_fmssf4 = 335,
  CODE_FOR_divsf3 = 343,
  CODE_FOR_negsf2 = 348,
  CODE_FOR_abssf2 = 353,
  CODE_FOR_sqrtsf2 = 357,
  CODE_FOR_ashlsi3 = 358,
  CODE_FOR_ashldi3_v8plus = 360,
  CODE_FOR_ashrsi3 = 363,
  CODE_FOR_ashrdi3_v8plus = 368,
  CODE_FOR_lshrsi3 = 369,
  CODE_FOR_lshrdi3_v8plus = 374,
  CODE_FOR_window_save = 388,
  CODE_FOR_eh_return_internal = 389,
  CODE_FOR_blockage = 391,
  CODE_FOR_speculation_barrier = 393,
   CODE_FOR_probe_stack_rangesi = CODE_FOR_nothing,
  CODE_FOR_probe_stack_rangedi = 394,
   CODE_FOR_update_return = CODE_FOR_nothing,
  CODE_FOR_nop = 395,
  CODE_FOR_flush_register_windows = 397,
   CODE_FOR_flushsi = CODE_FOR_nothing,
  CODE_FOR_flushdi = 398,
  CODE_FOR_ldfsr = 399,
  CODE_FOR_stfsr = 400,
   CODE_FOR_popcountdi_v8plus = CODE_FOR_nothing,
   CODE_FOR_popcountsi_v8plus = CODE_FOR_nothing,
   CODE_FOR_clzdi_v8plus = CODE_FOR_nothing,
   CODE_FOR_clzsi_v8plus = CODE_FOR_nothing,
  CODE_FOR_prefetch_64 = 405,
  CODE_FOR_prefetch_32 = 406,
  CODE_FOR_trap = 407,
   CODE_FOR_tgd_hi22si = CODE_FOR_nothing,
   CODE_FOR_tgd_hi22di = CODE_FOR_nothing,
   CODE_FOR_tgd_lo10si = CODE_FOR_nothing,
   CODE_FOR_tgd_lo10di = CODE_FOR_nothing,
   CODE_FOR_tgd_addsi = CODE_FOR_nothing,
   CODE_FOR_tgd_adddi = CODE_FOR_nothing,
   CODE_FOR_tgd_callsi = CODE_FOR_nothing,
   CODE_FOR_tgd_calldi = CODE_FOR_nothing,
   CODE_FOR_tldm_hi22si = CODE_FOR_nothing,
   CODE_FOR_tldm_hi22di = CODE_FOR_nothing,
   CODE_FOR_tldm_lo10si = CODE_FOR_nothing,
   CODE_FOR_tldm_lo10di = CODE_FOR_nothing,
   CODE_FOR_tldm_addsi = CODE_FOR_nothing,
   CODE_FOR_tldm_adddi = CODE_FOR_nothing,
   CODE_FOR_tldm_callsi = CODE_FOR_nothing,
   CODE_FOR_tldm_calldi = CODE_FOR_nothing,
   CODE_FOR_tldo_hix22si = CODE_FOR_nothing,
   CODE_FOR_tldo_hix22di = CODE_FOR_nothing,
   CODE_FOR_tldo_lox10si = CODE_FOR_nothing,
   CODE_FOR_tldo_lox10di = CODE_FOR_nothing,
   CODE_FOR_tldo_addsi = CODE_FOR_nothing,
   CODE_FOR_tldo_adddi = CODE_FOR_nothing,
   CODE_FOR_tie_hi22si = CODE_FOR_nothing,
   CODE_FOR_tie_hi22di = CODE_FOR_nothing,
   CODE_FOR_tie_lo10si = CODE_FOR_nothing,
   CODE_FOR_tie_lo10di = CODE_FOR_nothing,
   CODE_FOR_tie_ld32 = CODE_FOR_nothing,
   CODE_FOR_tie_ld64 = CODE_FOR_nothing,
   CODE_FOR_tie_addsi = CODE_FOR_nothing,
   CODE_FOR_tie_adddi = CODE_FOR_nothing,
   CODE_FOR_tle_hix22si = CODE_FOR_nothing,
   CODE_FOR_tle_hix22di = CODE_FOR_nothing,
   CODE_FOR_tle_lox10si = CODE_FOR_nothing,
   CODE_FOR_tle_lox10di = CODE_FOR_nothing,
   CODE_FOR_stack_protect_set32 = CODE_FOR_nothing,
  CODE_FOR_stack_protect_set64 = 410,
   CODE_FOR_stack_protect_test32 = CODE_FOR_nothing,
  CODE_FOR_stack_protect_test64 = 411,
  CODE_FOR_addv1si3 = 419,
  CODE_FOR_subv1si3 = 420,
  CODE_FOR_addv2si3 = 421,
  CODE_FOR_subv2si3 = 422,
  CODE_FOR_addv2hi3 = 423,
  CODE_FOR_subv2hi3 = 424,
  CODE_FOR_addv4hi3 = 425,
  CODE_FOR_subv4hi3 = 426,
  CODE_FOR_iorv1si3 = 427,
  CODE_FOR_andv1si3 = 428,
  CODE_FOR_xorv1si3 = 429,
  CODE_FOR_iorv2hi3 = 430,
  CODE_FOR_andv2hi3 = 431,
  CODE_FOR_xorv2hi3 = 432,
  CODE_FOR_iorv4qi3 = 433,
  CODE_FOR_andv4qi3 = 434,
  CODE_FOR_xorv4qi3 = 435,
  CODE_FOR_iorv1di3 = 436,
  CODE_FOR_andv1di3 = 437,
  CODE_FOR_xorv1di3 = 438,
  CODE_FOR_iorv2si3 = 439,
  CODE_FOR_andv2si3 = 440,
  CODE_FOR_xorv2si3 = 441,
  CODE_FOR_iorv4hi3 = 442,
  CODE_FOR_andv4hi3 = 443,
  CODE_FOR_xorv4hi3 = 444,
  CODE_FOR_iorv8qi3 = 445,
  CODE_FOR_andv8qi3 = 446,
  CODE_FOR_xorv8qi3 = 447,
  CODE_FOR_one_cmplv1si2 = 504,
  CODE_FOR_one_cmplv2hi2 = 505,
  CODE_FOR_one_cmplv4qi2 = 506,
  CODE_FOR_one_cmplv1di2 = 507,
  CODE_FOR_one_cmplv2si2 = 508,
  CODE_FOR_one_cmplv4hi2 = 509,
  CODE_FOR_one_cmplv8qi2 = 510,
  CODE_FOR_fpack16_vis = 511,
  CODE_FOR_fpackfix_vis = 512,
  CODE_FOR_fpack32_vis = 513,
  CODE_FOR_fexpand_vis = 514,
  CODE_FOR_fpmerge_vis = 515,
  CODE_FOR_fmul8x16_vis = 516,
  CODE_FOR_fmul8x16au_vis = 517,
  CODE_FOR_fmul8x16al_vis = 518,
  CODE_FOR_fmul8sux16_vis = 519,
  CODE_FOR_fmul8ulx16_vis = 520,
  CODE_FOR_fmuld8sux16_vis = 521,
  CODE_FOR_fmuld8ulx16_vis = 522,
   CODE_FOR_wrgsr_v8plus = CODE_FOR_nothing,
   CODE_FOR_rdgsr_v8plus = CODE_FOR_nothing,
  CODE_FOR_faligndatav1di_vis = 525,
  CODE_FOR_faligndatav2si_vis = 526,
  CODE_FOR_faligndatav4hi_vis = 527,
  CODE_FOR_faligndatav8qi_vis = 528,
  CODE_FOR_alignaddrsi_vis = 529,
  CODE_FOR_alignaddrdi_vis = 530,
  CODE_FOR_alignaddrlsi_vis = 531,
  CODE_FOR_alignaddrldi_vis = 532,
  CODE_FOR_pdist_vis = 533,
   CODE_FOR_edge8si_vis = CODE_FOR_nothing,
  CODE_FOR_edge8di_vis = 534,
   CODE_FOR_edge8lsi_vis = CODE_FOR_nothing,
  CODE_FOR_edge8ldi_vis = 535,
   CODE_FOR_edge16si_vis = CODE_FOR_nothing,
  CODE_FOR_edge16di_vis = 536,
   CODE_FOR_edge16lsi_vis = CODE_FOR_nothing,
  CODE_FOR_edge16ldi_vis = 537,
   CODE_FOR_edge32si_vis = CODE_FOR_nothing,
  CODE_FOR_edge32di_vis = 538,
   CODE_FOR_edge32lsi_vis = CODE_FOR_nothing,
  CODE_FOR_edge32ldi_vis = 539,
   CODE_FOR_fpcmpeq8si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpne8si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmple8si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpgt8si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpeq16si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpne16si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmple16si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpgt16si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpeq32si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpne32si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmple32si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpgt32si_vis = CODE_FOR_nothing,
  CODE_FOR_fpcmpeq8di_vis = 540,
  CODE_FOR_fpcmpne8di_vis = 541,
  CODE_FOR_fpcmple8di_vis = 542,
  CODE_FOR_fpcmpgt8di_vis = 543,
  CODE_FOR_fpcmpeq16di_vis = 544,
  CODE_FOR_fpcmpne16di_vis = 545,
  CODE_FOR_fpcmple16di_vis = 546,
  CODE_FOR_fpcmpgt16di_vis = 547,
  CODE_FOR_fpcmpeq32di_vis = 548,
  CODE_FOR_fpcmpne32di_vis = 549,
  CODE_FOR_fpcmple32di_vis = 550,
  CODE_FOR_fpcmpgt32di_vis = 551,
   CODE_FOR_fpcmpule8si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpugt8si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpule16si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpugt16si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpule32si_vis = CODE_FOR_nothing,
   CODE_FOR_fpcmpugt32si_vis = CODE_FOR_nothing,
  CODE_FOR_fpcmpule8di_vis = 552,
  CODE_FOR_fpcmpugt8di_vis = 553,
  CODE_FOR_fpcmpule16di_vis = 554,
  CODE_FOR_fpcmpugt16di_vis = 555,
  CODE_FOR_fpcmpule32di_vis = 556,
  CODE_FOR_fpcmpugt32di_vis = 557,
   CODE_FOR_array8si_vis = CODE_FOR_nothing,
  CODE_FOR_array8di_vis = 558,
   CODE_FOR_array16si_vis = CODE_FOR_nothing,
  CODE_FOR_array16di_vis = 559,
   CODE_FOR_array32si_vis = CODE_FOR_nothing,
  CODE_FOR_array32di_vis = 560,
  CODE_FOR_bmaskdi_vis = 561,
  CODE_FOR_bmasksi_vis = 562,
  CODE_FOR_bshufflev1di_vis = 563,
  CODE_FOR_bshufflev2si_vis = 564,
  CODE_FOR_bshufflev4hi_vis = 565,
  CODE_FOR_bshufflev8qi_vis = 566,
   CODE_FOR_edge8nsi_vis = CODE_FOR_nothing,
  CODE_FOR_edge8ndi_vis = 567,
   CODE_FOR_edge8lnsi_vis = CODE_FOR_nothing,
  CODE_FOR_edge8lndi_vis = 568,
   CODE_FOR_edge16nsi_vis = CODE_FOR_nothing,
  CODE_FOR_edge16ndi_vis = 569,
   CODE_FOR_edge16lnsi_vis = CODE_FOR_nothing,
  CODE_FOR_edge16lndi_vis = 570,
   CODE_FOR_edge32nsi_vis = CODE_FOR_nothing,
  CODE_FOR_edge32ndi_vis = 571,
   CODE_FOR_edge32lnsi_vis = CODE_FOR_nothing,
  CODE_FOR_edge32lndi_vis = 572,
   CODE_FOR_cmask8si_vis = CODE_FOR_nothing,
  CODE_FOR_cmask8di_vis = 573,
   CODE_FOR_cmask16si_vis = CODE_FOR_nothing,
  CODE_FOR_cmask16di_vis = 574,
   CODE_FOR_cmask32si_vis = CODE_FOR_nothing,
  CODE_FOR_cmask32di_vis = 575,
  CODE_FOR_fchksm16_vis = 576,
  CODE_FOR_vashlv4hi3 = 577,
  CODE_FOR_vssashlv4hi3 = 578,
  CODE_FOR_vlshrv4hi3 = 579,
  CODE_FOR_vashrv4hi3 = 580,
  CODE_FOR_vashlv2si3 = 581,
  CODE_FOR_vssashlv2si3 = 582,
  CODE_FOR_vlshrv2si3 = 583,
  CODE_FOR_vashrv2si3 = 584,
   CODE_FOR_pdistnsi_vis = CODE_FOR_nothing,
  CODE_FOR_pdistndi_vis = 585,
  CODE_FOR_fmean16_vis = 586,
  CODE_FOR_fpadd64_vis = 587,
  CODE_FOR_fpsub64_vis = 588,
  CODE_FOR_addv8qi3 = 589,
  CODE_FOR_subv8qi3 = 590,
  CODE_FOR_ssaddv4hi3 = 591,
  CODE_FOR_sssubv4hi3 = 592,
  CODE_FOR_ssaddv2si3 = 593,
  CODE_FOR_sssubv2si3 = 594,
  CODE_FOR_ssaddv2hi3 = 595,
  CODE_FOR_sssubv2hi3 = 596,
  CODE_FOR_ssaddv1si3 = 597,
  CODE_FOR_sssubv1si3 = 598,
  CODE_FOR_minv8qi3 = 599,
  CODE_FOR_maxv8qi3 = 600,
  CODE_FOR_minv4hi3 = 601,
  CODE_FOR_maxv4hi3 = 602,
  CODE_FOR_minv2si3 = 603,
  CODE_FOR_maxv2si3 = 604,
  CODE_FOR_minuv8qi3 = 605,
  CODE_FOR_maxuv8qi3 = 606,
  CODE_FOR_minuv4hi3 = 607,
  CODE_FOR_maxuv4hi3 = 608,
  CODE_FOR_minuv2si3 = 609,
  CODE_FOR_maxuv2si3 = 610,
  CODE_FOR_ssaddv8qi3 = 611,
  CODE_FOR_sssubv8qi3 = 612,
  CODE_FOR_usaddv4hi3 = 613,
  CODE_FOR_ussubv4hi3 = 614,
  CODE_FOR_usaddv8qi3 = 615,
  CODE_FOR_ussubv8qi3 = 616,
  CODE_FOR_fhaddsf_vis = 622,
  CODE_FOR_fhadddf_vis = 623,
  CODE_FOR_fhsubsf_vis = 624,
  CODE_FOR_fhsubdf_vis = 625,
  CODE_FOR_fnhaddsf_vis = 626,
  CODE_FOR_fnhadddf_vis = 627,
  CODE_FOR_dictunpack8 = 628,
  CODE_FOR_dictunpack16 = 629,
  CODE_FOR_dictunpack32 = 630,
   CODE_FOR_fpcmpeq8sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpne8sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmple8sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpgt8sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpeq16sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpne16sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmple16sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpgt16sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpeq32sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpne32sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmple32sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpgt32sishl = CODE_FOR_nothing,
  CODE_FOR_fpcmpeq8dishl = 631,
  CODE_FOR_fpcmpne8dishl = 632,
  CODE_FOR_fpcmple8dishl = 633,
  CODE_FOR_fpcmpgt8dishl = 634,
  CODE_FOR_fpcmpeq16dishl = 635,
  CODE_FOR_fpcmpne16dishl = 636,
  CODE_FOR_fpcmple16dishl = 637,
  CODE_FOR_fpcmpgt16dishl = 638,
  CODE_FOR_fpcmpeq32dishl = 639,
  CODE_FOR_fpcmpne32dishl = 640,
  CODE_FOR_fpcmple32dishl = 641,
  CODE_FOR_fpcmpgt32dishl = 642,
   CODE_FOR_fpcmpule8sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpugt8sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpule16sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpugt16sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpule32sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpugt32sishl = CODE_FOR_nothing,
  CODE_FOR_fpcmpule8dishl = 643,
  CODE_FOR_fpcmpugt8dishl = 644,
  CODE_FOR_fpcmpule16dishl = 645,
  CODE_FOR_fpcmpugt16dishl = 646,
  CODE_FOR_fpcmpule32dishl = 647,
  CODE_FOR_fpcmpugt32dishl = 648,
   CODE_FOR_fpcmpde8sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpde16sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpde32sishl = CODE_FOR_nothing,
  CODE_FOR_fpcmpde8dishl = 649,
  CODE_FOR_fpcmpde16dishl = 650,
  CODE_FOR_fpcmpde32dishl = 651,
   CODE_FOR_fpcmpur8sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpur16sishl = CODE_FOR_nothing,
   CODE_FOR_fpcmpur32sishl = CODE_FOR_nothing,
  CODE_FOR_fpcmpur8dishl = 652,
  CODE_FOR_fpcmpur16dishl = 653,
  CODE_FOR_fpcmpur32dishl = 654,
  CODE_FOR_membar_storeload = 657,
   CODE_FOR_atomic_loaddi_1 = CODE_FOR_nothing,
   CODE_FOR_atomic_storedi_1 = CODE_FOR_nothing,
  CODE_FOR_atomic_compare_and_swap_leon3_1 = 662,
  CODE_FOR_swapsi = 664,
  CODE_FOR_ldstub = 665,
  CODE_FOR_cstoresi4 = 666,
  CODE_FOR_cstoredi4 = 667,
  CODE_FOR_cstoresf4 = 668,
  CODE_FOR_cstoredf4 = 669,
  CODE_FOR_cstoretf4 = 670,
  CODE_FOR_cbranchcc4 = 671,
  CODE_FOR_cbranchsi4 = 672,
  CODE_FOR_cbranchdi4 = 673,
  CODE_FOR_cbranchsf4 = 674,
  CODE_FOR_cbranchdf4 = 675,
  CODE_FOR_cbranchtf4 = 676,
  CODE_FOR_movqi = 677,
  CODE_FOR_movhi = 678,
  CODE_FOR_movsi = 679,
  CODE_FOR_movsi_pic_label_ref = 680,
   CODE_FOR_vxworks_load_got = CODE_FOR_nothing,
  CODE_FOR_movdi = 681,
  CODE_FOR_movdi_pic_label_ref = 682,
  CODE_FOR_reload_indi = 683,
  CODE_FOR_reload_outdi = 684,
  CODE_FOR_movti = 685,
  CODE_FOR_movsf = 686,
  CODE_FOR_movdf = 687,
  CODE_FOR_movtf = 688,
  CODE_FOR_movqicc = 689,
  CODE_FOR_movhicc = 690,
  CODE_FOR_movsicc = 691,
  CODE_FOR_movdicc = 692,
  CODE_FOR_movsfcc = 693,
  CODE_FOR_movdfcc = 694,
  CODE_FOR_movtfcc = 695,
  CODE_FOR_zero_extendhisi2 = 696,
  CODE_FOR_zero_extendqihi2 = 697,
  CODE_FOR_zero_extendqisi2 = 698,
  CODE_FOR_zero_extendqidi2 = 699,
  CODE_FOR_zero_extendhidi2 = 700,
  CODE_FOR_zero_extendsidi2 = 701,
  CODE_FOR_extendhisi2 = 702,
  CODE_FOR_extendqihi2 = 703,
  CODE_FOR_extendqisi2 = 704,
  CODE_FOR_extendqidi2 = 705,
  CODE_FOR_extendhidi2 = 706,
  CODE_FOR_extendsidi2 = 707,
  CODE_FOR_extendsftf2 = 708,
  CODE_FOR_extenddftf2 = 709,
  CODE_FOR_trunctfsf2 = 710,
  CODE_FOR_trunctfdf2 = 711,
  CODE_FOR_floatsitf2 = 712,
  CODE_FOR_floatunssitf2 = 713,
  CODE_FOR_floatunsdisf2 = 714,
  CODE_FOR_floatunsdidf2 = 715,
  CODE_FOR_floatditf2 = 716,
  CODE_FOR_floatunsditf2 = 717,
  CODE_FOR_fix_trunctfsi2 = 718,
  CODE_FOR_fixuns_trunctfsi2 = 719,
  CODE_FOR_fixuns_truncsfdi2 = 720,
  CODE_FOR_fixuns_truncdfdi2 = 721,
  CODE_FOR_fix_trunctfdi2 = 722,
  CODE_FOR_fixuns_trunctfdi2 = 723,
  CODE_FOR_adddi3 = 724,
  CODE_FOR_uaddvdi4 = 725,
  CODE_FOR_addvdi4 = 726,
  CODE_FOR_uaddvsi4 = 727,
  CODE_FOR_addvsi4 = 728,
  CODE_FOR_subdi3 = 729,
  CODE_FOR_usubvdi4 = 730,
  CODE_FOR_subvdi4 = 731,
  CODE_FOR_usubvsi4 = 732,
  CODE_FOR_subvsi4 = 733,
  CODE_FOR_mulsi3 = 734,
  CODE_FOR_muldi3 = 735,
  CODE_FOR_mulsidi3 = 736,
   CODE_FOR_smulsi3_highpart = CODE_FOR_nothing,
  CODE_FOR_umulsidi3 = 737,
   CODE_FOR_umulsi3_highpart = CODE_FOR_nothing,
  CODE_FOR_umulxhi_vis = 738,
  CODE_FOR_xmulx_vis = 739,
  CODE_FOR_xmulxhi_vis = 740,
  CODE_FOR_divsi3 = 741,
  CODE_FOR_udivsi3 = 742,
  CODE_FOR_negdi2 = 743,
  CODE_FOR_unegvdi3 = 744,
  CODE_FOR_negvdi3 = 745,
  CODE_FOR_unegvsi3 = 746,
  CODE_FOR_negvsi3 = 747,
  CODE_FOR_addtf3 = 748,
  CODE_FOR_subtf3 = 749,
  CODE_FOR_multf3 = 750,
  CODE_FOR_divtf3 = 751,
  CODE_FOR_divdf3 = 752,
  CODE_FOR_negtf2 = 753,
  CODE_FOR_negdf2 = 754,
  CODE_FOR_abstf2 = 755,
  CODE_FOR_absdf2 = 756,
  CODE_FOR_sqrttf2 = 757,
  CODE_FOR_sqrtdf2 = 758,
  CODE_FOR_ashldi3 = 759,
  CODE_FOR_ashrdi3 = 760,
  CODE_FOR_lshrdi3 = 761,
  CODE_FOR_jump = 762,
  CODE_FOR_tablejump = 763,
  CODE_FOR_call = 764,
  CODE_FOR_call_value = 765,
  CODE_FOR_untyped_call = 766,
  CODE_FOR_sibcall = 767,
  CODE_FOR_sibcall_value = 768,
  CODE_FOR_prologue = 769,
  CODE_FOR_epilogue = 770,
  CODE_FOR_sibcall_epilogue = 771,
  CODE_FOR_eh_return = 772,
  CODE_FOR_return = 773,
  CODE_FOR_frame_blockage = 774,
  CODE_FOR_probe_stack = 775,
  CODE_FOR_untyped_return = 776,
  CODE_FOR_indirect_jump = 777,
  CODE_FOR_save_stack_nonlocal = 778,
  CODE_FOR_restore_stack_nonlocal = 779,
  CODE_FOR_nonlocal_goto = 780,
   CODE_FOR_builtin_setjmp_receiver = CODE_FOR_nothing,
  CODE_FOR_popcountdi2 = 781,
  CODE_FOR_popcountsi2 = 782,
  CODE_FOR_clzdi2 = 783,
  CODE_FOR_clzsi2 = 784,
  CODE_FOR_prefetch = 785,
  CODE_FOR_ctrapsi4 = 786,
  CODE_FOR_ctrapdi4 = 787,
  CODE_FOR_stack_protect_set = 788,
  CODE_FOR_stack_protect_test = 789,
  CODE_FOR_movv1si = 790,
  CODE_FOR_movv2hi = 791,
  CODE_FOR_movv4qi = 792,
  CODE_FOR_movv1di = 793,
  CODE_FOR_movv2si = 794,
  CODE_FOR_movv4hi = 795,
  CODE_FOR_movv8qi = 796,
  CODE_FOR_vec_initv1sisi = 797,
  CODE_FOR_vec_initv2hihi = 798,
  CODE_FOR_vec_initv4qiqi = 799,
  CODE_FOR_vec_initv1didi = 800,
  CODE_FOR_vec_initv2sisi = 801,
  CODE_FOR_vec_initv4hihi = 802,
  CODE_FOR_vec_initv8qiqi = 803,
  CODE_FOR_wrgsr_vis = 804,
  CODE_FOR_rdgsr_vis = 805,
   CODE_FOR_vec_cmpv8qisi = CODE_FOR_nothing,
   CODE_FOR_vec_cmpv4hisi = CODE_FOR_nothing,
   CODE_FOR_vec_cmpv2sisi = CODE_FOR_nothing,
  CODE_FOR_vec_cmpv8qidi = 806,
  CODE_FOR_vec_cmpv4hidi = 807,
  CODE_FOR_vec_cmpv2sidi = 808,
   CODE_FOR_vec_cmpuv8qisi = CODE_FOR_nothing,
   CODE_FOR_vec_cmpuv4hisi = CODE_FOR_nothing,
   CODE_FOR_vec_cmpuv2sisi = CODE_FOR_nothing,
  CODE_FOR_vec_cmpuv8qidi = 809,
  CODE_FOR_vec_cmpuv4hidi = 810,
  CODE_FOR_vec_cmpuv2sidi = 811,
   CODE_FOR_vcond_mask_v8qisi = CODE_FOR_nothing,
   CODE_FOR_vcond_mask_v4hisi = CODE_FOR_nothing,
   CODE_FOR_vcond_mask_v2sisi = CODE_FOR_nothing,
  CODE_FOR_vcond_mask_v8qidi = 812,
  CODE_FOR_vcond_mask_v4hidi = 813,
  CODE_FOR_vcond_mask_v2sidi = 814,
  CODE_FOR_vec_permv1di = 815,
  CODE_FOR_vec_permv2si = 816,
  CODE_FOR_vec_permv4hi = 817,
  CODE_FOR_vec_permv8qi = 818,
  CODE_FOR_mem_thread_fence = 819,
  CODE_FOR_membar = 820,
  CODE_FOR_atomic_loadqi = 821,
  CODE_FOR_atomic_loadhi = 822,
  CODE_FOR_atomic_loadsi = 823,
  CODE_FOR_atomic_loaddi = 824,
  CODE_FOR_atomic_storeqi = 825,
  CODE_FOR_atomic_storehi = 826,
  CODE_FOR_atomic_storesi = 827,
  CODE_FOR_atomic_storedi = 828,
  CODE_FOR_atomic_compare_and_swapqi = 829,
  CODE_FOR_atomic_compare_and_swaphi = 830,
  CODE_FOR_atomic_compare_and_swapsi = 831,
  CODE_FOR_atomic_compare_and_swapdi = 832,
  CODE_FOR_atomic_compare_and_swapsi_1 = 833,
  CODE_FOR_atomic_compare_and_swapdi_1 = 834,
  CODE_FOR_atomic_exchangesi = 835,
  CODE_FOR_atomic_test_and_set = 836
};

const unsigned int NUM_INSN_CODES = 837;
#endif /* GCC_INSN_CODES_H */
