/* Generated automatically. */
static const char configuration_arguments[] = "/usr/obj/ports/gcc-15.2.0/gcc-15.2.0/configure --build=sparc64-unknown-openbsd --with-stage1-ldflags=-L/usr/obj/ports/gcc-15.2.0/bootstrap/lib --verbose --program-transform-name='s,^,e,' --disable-nls --with-system-zlib --disable-libmudflap --disable-libgomp --disable-libssp --disable-tls --with-gnu-ld --with-gnu-as --enable-threads=posix --enable-wchar_t --with-gmp=/usr/local --enable-languages=c,c++,fortran,objc,d --disable-libstdcxx-pch --enable-standard-branch-protection --without-isl --enable-default-ssp --enable-host-pie --enable-default-pie --enable-cpp --prefix=/usr/local --sysconfdir=/etc --mandir=/usr/local/man --infodir=/usr/local/info --localstatedir=/var --disable-silent-rules --disable-gtk-doc";
static const char thread_model[] = "posix";

static const struct {
  const char *name, *value;
} configure_default_options[] = { { "cpu", "ultrasparc" } };
